diff --git a/ieee80211_node.c b/ieee80211_node.c
index 861fa85..a609cb1 100644
--- a/ieee80211_node.c
+++ b/ieee80211_node.c
@@ -2156,32 +2156,79 @@ ieee80211_node_timeout(void *arg)
 		ieee80211_node_timeout, ic);
 }
 
+/*
+ * May directly be called and do customized iterate functions.
+ * Only requirement is to decrement each node's ref count.
+ */
 void
-ieee80211_iterate_nodes(struct ieee80211_node_table *nt,
-	ieee80211_iter_func *f, void *arg)
+ieee80211_iterate_nt(struct ieee80211_node_table *nt,
+    struct ieee80211_node **ni_arr, uint16_t max_aid)
 {
 	struct ieee80211_node *ni;
 	u_int gen;
+	int i = 0;
 
 	IEEE80211_NODE_ITERATE_LOCK(nt);
+	IEEE80211_NODE_LOCK(nt);
+
 	gen = ++nt->nt_scangen;
+
 restart:
-	IEEE80211_NODE_LOCK(nt);
 	TAILQ_FOREACH(ni, &nt->nt_node, ni_list) {
-		if (ni->ni_scangen != gen) {
-			ni->ni_scangen = gen;
-			(void) ieee80211_ref_node(ni);
-			IEEE80211_NODE_UNLOCK(nt);
-			(*f)(arg, ni);
-			ieee80211_free_node(ni);
-			goto restart;
+		if (ni->ni_scangen == gen)
+			continue;
+
+		if (i >= max_aid) {
+			if_printf(nt->nt_ic->ic_ifp,
+			    "Node array overflow: max=%u", max_aid);
+			break;
 		}
+
+		ni->ni_scangen = gen;
+		(*(ni_arr + i++)) = ieee80211_ref_node(ni);
+		goto restart;
 	}
-	IEEE80211_NODE_UNLOCK(nt);
 
+	IEEE80211_NODE_UNLOCK(nt);
 	IEEE80211_NODE_ITERATE_UNLOCK(nt);
 }
 
+/*
+ * Just a wrapper, so we don't have to change every ieee80211_iterate_nodes()
+ * reference in the source.
+ */
+void
+ieee80211_iterate_nodes(struct ieee80211_node_table *nt,
+	ieee80211_iter_func *f, void *arg)
+{
+	struct ieee80211_node **ni_arr;
+	struct ieee80211_node *ni;
+	unsigned long size;
+	int i;
+	uint16_t max_aid;
+
+	max_aid = TAILQ_FIRST(&nt->nt_ic->ic_vaps)->iv_max_aid;
+	size = max_aid * sizeof(*ni_arr);
+	ni_arr = (struct ieee80211_node **)malloc(size, M_80211_NODE,
+	    M_NOWAIT | M_ZERO);
+	if (ni_arr == NULL)
+		return;
+
+	ieee80211_iterate_nt(nt, ni_arr, max_aid);
+
+	for (i = 0; i < max_aid; i++) {
+		ni = *(ni_arr + i);
+		if (ni == NULL)	/* end of the list */
+			break;
+
+		(*f)(arg, ni);
+		/* ieee80211_free_node() locks by itself */
+		ieee80211_free_node(ni);
+	}
+
+	free(ni_arr, M_80211_NODE);
+}
+
 void
 ieee80211_dump_node(struct ieee80211_node_table *nt, struct ieee80211_node *ni)
 {
diff --git a/ieee80211_node.h b/ieee80211_node.h
index 83b108b..1fbc892 100644
--- a/ieee80211_node.h
+++ b/ieee80211_node.h
@@ -438,6 +438,8 @@ int	ieee80211_node_delucastkey(struct ieee80211_node *);
 void	ieee80211_node_timeout(void *arg);
 
 typedef void ieee80211_iter_func(void *, struct ieee80211_node *);
+void	ieee80211_iterate_nt(struct ieee80211_node_table *,
+		struct ieee80211_node **, uint16_t);
 void	ieee80211_iterate_nodes(struct ieee80211_node_table *,
 		ieee80211_iter_func *, void *);
 